@@ -17,6 +17,7 @@ from string import Template
 import click
 import discord
 import dunamai as _dunamai
+import hupper
 import matplotlib
 import matplotlib.dates as mdates
 import matplotlib.pyplot as plt
@@ -83,6 +84,16 @@ COLLECTABLE_SET = FOSSILS_SET | FISH_SET | BUGS_SET | ART_SET
 
 EMBED_LIMIT = 5  # more embeds in a row than this causes issues
 
+USER_PREFRENCES = [
+    "hemisphere",
+    "timezone",
+    "island",
+    "friend",
+    "fruit",
+    "nickname",
+    "creator",
+]
+
 DAYS = {
     "sunday": 0,
     "monday": 1,
@@ -94,6 +105,43 @@ DAYS = {
 }
 
 
+class Validate:
+    FRUITS = ["apple", "cherry", "orange", "peach", "pear"]
+    HEMISPHERES = ["northern", "southern"]
+
+    @classmethod
+    def friend(cls, value):
+        code = re.sub("[^0-9]", "", value)
+        return code if len(code) == 12 and code.isdigit() else None
+
+    @classmethod
+    def creator(cls, value):
+        code = re.sub("[^0-9]", "", value)
+        return code if len(code) == 12 and code.isdigit() else None
+
+    @classmethod
+    def fruit(cls, value):
+        fruit = value.lower()
+        return fruit if fruit in cls.FRUITS else None
+
+    @classmethod
+    def hemisphere(cls, value):
+        home = value.lower()
+        return home if home in cls.HEMISPHERES else None
+
+    @classmethod
+    def nickname(cls, value):
+        return value
+
+    @classmethod
+    def timezone(cls, value):
+        return value if value in pytz.all_timezones_set else None
+
+    @classmethod
+    def island(cls, value):
+        return value
+
+
 def s(key, **kwargs):
     """Returns a string from data/strings.yaml with subsitutions."""
     data = STRINGS.get(key, "")
@@ -177,6 +225,8 @@ def discord_user_from_name(channel, name):
 
 def discord_user_from_id(channel, user_id):
     """Returns the discord user from the given channel and user id."""
+    if user_id is None:
+        return None
     iid = int(user_id)
     members = channel.members
     return next(filter(lambda member: iid == member.id, members), None)
@@ -211,17 +261,22 @@ def is_turbot_admin(channel, user_or_member):
     return any(role.name == "Turbot Admin" for role in member.roles) if member else False
 
 
+def command(f):
+    f.is_command = True
+    return f
+
+
 class Turbot(discord.Client):
     """Discord turnip bot"""
 
     def __init__(
         self,
-        token,
-        channels,
-        prices_file,
-        art_file,
-        fossils_file,
-        users_file,
+        token="",
+        channels=[],
+        prices_file=DEFAULT_DB_PRICES,
+        art_file=DEFAULT_DB_ART,
+        fossils_file=DEFAULT_DB_FOSSILS,
+        users_file=DEFAULT_DB_USERS,
         log_level=None,
     ):
         if log_level:  # pragma: no cover
@@ -240,6 +295,14 @@ class Turbot(discord.Client):
         self._users_data = None  # do not use directly, load it from load_users()
         self._last_backup_filename = None
 
+        # build a list of commands supported by this bot by fetching @command methods
+        members = inspect.getmembers(self, predicate=inspect.ismethod)
+        self._commands = [
+            member[0]
+            for member in members
+            if hasattr(member[1], "is_command") and member[1].is_command
+        ]
+
     def run(self):  # pragma: no cover
         super().run(self.token)
 
@@ -286,21 +349,20 @@ class Turbot(discord.Client):
 
     def load_users(self):
         """Returns a DataFrame of user data or creates an empty one."""
-        if self._users_data is None:
-            try:
-                self._users_data = pd.read_csv(self.users_file)
-            except FileNotFoundError:
-                self._users_data = pd.DataFrame(
-                    columns=[
-                        "author",
-                        "hemisphere",
-                        "timezone",
-                        "island",
-                        "friend",
-                        "fruit",
-                        "nickname",
-                    ]
-                )
+        if self._users_data is not None:
+            self._users_data = self._users_data.fillna("")
+            return self._users_data
+
+        cols = ["author", *USER_PREFRENCES]
+        dtypes = ["int64", "str", "str", "str", "str", "str", "str", "str"]
+        if Path(self.users_file).exists():
+            self._users_data = pd.read_csv(self.users_file, names=cols, skiprows=1)
+        else:
+            self._users_data = pd.read_csv(
+                StringIO(""), names=cols, dtype=dict(zip(cols, dtypes))
+            )
+        self._users_data = self._users_data.fillna("")
+        self._users_data = self._users_data.astype(dict(zip(cols, dtypes)))
         return self._users_data
 
     def save_art(self, data):
@@ -369,6 +431,23 @@ class Turbot(discord.Client):
         plot_models_range(
             island.name, list(island.model_group.models), island.previous_week, True
         )
+
+        # fit the y-axis to the plotted data
+        maximum = 0
+        ax = plt.gca()
+        for line in ax.lines:
+            for price in line.get_ydata():
+                if price > maximum:
+                    maximum = price
+        for collection in ax.collections:
+            for point in collection.get_offsets():
+                _, y = point
+                if y > maximum:
+                    maximum = y
+        ax.set_ylim(0, maximum + 50)
+        ax.autoscale_view()
+        plt.draw()
+
         plt.savefig(graphname)
         return plt
 
@@ -462,14 +541,16 @@ class Turbot(discord.Client):
         row = users[users.author == user_id].tail(1)
         if row.empty:
             return {}
-        data = row.to_dict(orient="records")[0]
+
         prefs = {}
-        for column in users.columns:
-            if data and column in data and data[column] and isinstance(data[column], str):
-                if column == "timezone":
-                    prefs[column] = pytz.timezone(data[column])
-                else:
-                    prefs[column] = data[column]
+        data = row.to_dict(orient="records")[0]
+        for column in users.columns[1:]:
+            if not data[column]:
+                continue
+            if column == "timezone":
+                prefs[column] = pytz.timezone(data[column])
+            else:
+                prefs[column] = data[column]
         return prefs
 
     def get_user_timeline(self, user_id):
@@ -483,32 +564,41 @@ class Turbot(discord.Client):
         yours["timestamp"] = yours.timestamp.dt.tz_convert(target_timezone)
 
         recent_buy = yours[yours.kind == "buy"].tail(1)
-        if recent_buy.empty:
+        if recent_buy.empty:  # no buy found
             return [None] * 13
 
         buy_date = recent_buy.timestamp.iloc[0]
+        if buy_date.to_pydatetime().isoweekday() != 7:  # buy isn't on a sunday
+            return [None] * 13
+
         buy_price = int(recent_buy.price.iloc[0])
+        sells = yours[(yours.kind == "sell") & (yours.timestamp > buy_date)]
+        if sells.empty:  # no sells found after buy date
+            return [buy_price] + [None] * 12
+
+        sell_data = [
+            [None, None],  # monday
+            [None, None],  # tuesday
+            [None, None],  # wednesday
+            [None, None],  # friday
+            [None, None],  # thrusday
+            [None, None],  # saturday
+        ]
 
-        sells = yours[yours.kind == "sell"]
         groups = sells.set_index("timestamp").groupby(pd.Grouper(freq="D"))
-        sell_data = {}
         for day, df in groups:
-            days_since_buy = (day - buy_date).days
-            sell_data[days_since_buy] = df.price.iloc[0:2]
+            day_of_week = day.to_pydatetime().isoweekday()
+            if day_of_week == 7:  # no sells allowed on sundays
+                continue
+            for ts, row in df.iterrows():
+                if ts.hour < 12:  # am
+                    sell_data[day_of_week - 1][0] = int(row["price"])
+                else:  # pm
+                    sell_data[day_of_week - 1][1] = int(row["price"])
 
         timeline = [buy_price]
-        for day in range(0, 6):
-            if day in sell_data and sell_data[day].any():
-                am_price = int(sell_data[day][0])
-                timeline.append(am_price)
-                if len(sell_data[day]) > 1:
-                    pm_price = int(sell_data[day][1])
-                    timeline.append(pm_price)
-                else:
-                    timeline.append(None)
-            else:
-                timeline.extend([None, None])
-
+        for day in sell_data:
+            timeline.extend(day)
         return timeline
 
     def to_usertime(self, author_id, dt):
@@ -559,6 +649,11 @@ class Turbot(discord.Client):
 
         yield remaining
 
+    @property
+    def commands(self):
+        """Returns a list of commands supported by this bot."""
+        return self._commands
+
     async def process(self, message):
         """Process a command message."""
         tokens = message.content.split(" ")
@@ -566,18 +661,15 @@ class Turbot(discord.Client):
         params = list(filter(None, params))  # ignore any empty string parameters
         if not request:
             return
-        members = inspect.getmembers(self, predicate=inspect.ismethod)
-        commands = [member[0] for member in members if member[0].endswith("_command")]
-        matching = [command for command in commands if command.startswith(request)]
+        matching = [command for command in self.commands if command.startswith(request)]
         if not matching:
             await message.channel.send(s("not_a_command", request=request), file=None)
             return
-        exact = f"{request}_command"
-        if len(matching) > 1 and exact not in matching:
-            possible = ", ".join(f"!{m.replace('_command', '')}" for m in matching)
+        if len(matching) > 1 and request not in matching:
+            possible = ", ".join(f"!{m}" for m in matching)
             await message.channel.send(s("did_you_mean", possible=possible), file=None)
         else:
-            command = exact if exact in matching else matching[0]
+            command = request if request in matching else matching[0]
             logging.debug("%s (author=%s, params=%s)", command, message.author, params)
             method = getattr(self, command)
             async with message.channel.typing():
@@ -624,10 +716,11 @@ class Turbot(discord.Client):
     # Bot Command Functions
     ##############################
 
-    # Any method of this class with a name that ends in _command is automatically
-    # detected as a bot command. These methods should have a signature like:
+    # Any method of this class with a name that is decorated by @command is detected as a
+    # bot command. These methods should have a signature like:
     #
-    #     def your_command(self, channel, author, params)
+    #     @command
+    #     def command_name(self, channel, author, params)
     #
     # - `channel` is the Discord channel where the command message was sent.
     # - `author` is the Discord author who sent the command.
@@ -655,20 +748,20 @@ class Turbot(discord.Client):
     #
     # A [parameter] is optional whereas a <parameter> is required.
 
-    def help_command(self, channel, author, params):
+    @command
+    def help(self, channel, author, params):
         """
         Shows this help screen.
         """
-        members = inspect.getmembers(self, predicate=inspect.ismethod)
-        commands = [member[1] for member in members if member[0].endswith("_command")]
         usage = "__**Turbot Help!**__"
-        for command in commands:
-            doc = command.__doc__.split("|")
+        for command in self.commands:
+            method = getattr(self, command)
+            doc = method.__doc__.split("|")
             use, params = doc[0], ", ".join([param.strip() for param in doc[1:]])
             use = inspect.cleandoc(use)
             use = use.replace("\n", " ")
 
-            title = f"!{command.__name__.replace('_command', '')}"
+            title = f"!{command}"
             if params:
                 title = f"{title} {params}"
             usage += f"\n> **{title}**"
@@ -703,7 +796,8 @@ class Turbot(discord.Client):
         hour_offset = 13 if time_of_day == "evening" else 0
         return start + timedelta(days=day_offset, hours=hour_offset)
 
-    def sell_command(self, channel, author, params):
+    @command
+    def sell(self, channel, author, params):
         """
         Log the price that you can sell turnips for on your island.
         | <price> [day time]
@@ -740,7 +834,8 @@ class Turbot(discord.Client):
         )
         return s(key, price=price, name=author, last_price=last_price), None
 
-    def buy_command(self, channel, author, params):
+    @command
+    def buy(self, channel, author, params):
         """
         Log the price that you can buy turnips from Daisy Mae on your island.
         | <price> [day time]
@@ -766,7 +861,8 @@ class Turbot(discord.Client):
 
         return s("buy", price=price, name=author), None
 
-    def reset_command(self, channel, author, params):
+    @command
+    def reset(self, channel, author, params):
         """
         Only Turbot Admin members can run this command. Generates a final graph for use
         with !lastweek and resets all data for all users.
@@ -784,7 +880,8 @@ class Turbot(discord.Client):
         self.save_prices(prices)
         return s("reset"), None
 
-    def lastweek_command(self, channel, author, params):
+    @command
+    def lastweek(self, channel, author, params):
         """
         Displays the final graph from the last week before the data was reset.
         """
@@ -792,62 +889,16 @@ class Turbot(discord.Client):
             return s("lastweek_none"), None
         return s("lastweek"), discord.File(LASTWEEKCMD_FILE)
 
-    def graph_command(self, channel, author, params):
+    @command
+    def graph(self, channel, author, params):
         """
-        Generates a graph of turnip prices for all users. If a user is specified, only
-        graph that users prices. | [user]
+        Generates a historical graph of turnip prices for all users.
         """
+        self.generate_graph(channel, None, GRAPHCMD_FILE)
+        return s("graph_all_users"), discord.File(GRAPHCMD_FILE)
 
-        if not params:
-            self.generate_graph(channel, None, GRAPHCMD_FILE)
-            return s("graph_all_users"), discord.File(GRAPHCMD_FILE)
-
-        user_id = discord_user_id(channel, params[0])
-        user_name = discord_user_name(channel, user_id)
-        user = discord_user_from_name(channel, user_name)
-        if not user:
-            return s("cant_find_user", name=params[0]), None
-
-        self.generate_graph(channel, user, GRAPHCMD_FILE)
-        return s("graph_user", name=user_name), discord.File(GRAPHCMD_FILE)
-
-    def turnippattern_command(self, channel, author, params):
-        """
-        Calculates the patterns you will see in your shop based on Daisy Mae's price
-        on your island and your Monday morning sell price. |
-        <Sunday Buy Price> <Monday Morning Sell Price>
-        """
-        if len(params) != 2:
-            return s("turnippattern_bad_params"), None
-
-        buyprice, mondayprice = params
-        if not buyprice.isnumeric() or not mondayprice.isnumeric():
-            return s("turnippattern_nonnumeric_price"), None
-
-        buyprice, mondayprice = int(buyprice), int(mondayprice)
-        xval = mondayprice / buyprice
-        patterns = (
-            [1, 4]
-            if xval >= 0.91
-            else [2, 3, 4]
-            if xval >= 0.85
-            else [3, 4]
-            if xval >= 0.80
-            else [1, 4]
-            if xval >= 0.60
-            else [4]
-        )
-        lines = [s("turnippattern_header")]
-        if 1 in patterns:
-            lines.append(s("turnippattern_pattern1"))
-        if 2 in patterns:
-            lines.append(s("turnippattern_pattern2"))
-        if 3 in patterns:
-            lines.append(s("turnippattern_pattern3"))
-        lines.append(s("turnippattern_pattern4"))  # pattern 4 is always possible
-        return "\n".join(lines), None
-
-    def history_command(self, channel, author, params):
+    @command
+    def history(self, channel, author, params):
         """
         Show the historical turnip prices for a user. If no user is specified, it will
         display your own prices. | [user]
@@ -871,7 +922,8 @@ class Turbot(discord.Client):
             )
         return "\n".join(lines), None
 
-    def oops_command(self, channel, author, params):
+    @command
+    def oops(self, channel, author, params):
         """
         Remove your last logged turnip price.
         """
@@ -882,7 +934,8 @@ class Turbot(discord.Client):
         self.save_prices(prices)
         return s("oops", name=target_name), None
 
-    def clear_command(self, channel, author, params):
+    @command
+    def clear(self, channel, author, params):
         """
         Clears all of your own historical turnip prices.
         """
@@ -911,19 +964,22 @@ class Turbot(discord.Client):
             )
         return "\n".join(lines), None
 
-    def bestbuy_command(self, channel, author, params):
+    @command
+    def bestbuy(self, channel, author, params):
         """
         Finds the best (and most recent) buying prices logged in the last 12 hours.
         """
         return self._best(channel, author, "buy")
 
-    def bestsell_command(self, channel, author, params):
+    @command
+    def bestsell(self, channel, author, params):
         """
         Finds the best (and most recent) selling prices logged in the last 12 hours.
         """
         return self._best(channel, author, "sell")
 
-    def collect_command(self, channel, author, params):
+    @command
+    def collect(self, channel, author, params):
         """
         Mark collectables as donated to your museum. The names must match the in-game item
         name exactly. | <comma, separated, list, of, things>
@@ -986,7 +1042,8 @@ class Turbot(discord.Client):
 
         return "\n".join(lines), None
 
-    def uncollect_command(self, channel, author, params):
+    @command
+    def uncollect(self, channel, author, params):
         """
         Unmark collectables as donated to your museum. The names must match the in-game
         item name exactly. | <comma, separated, list, of, things>
@@ -1047,7 +1104,8 @@ class Turbot(discord.Client):
 
         return "\n".join(lines), None
 
-    def search_command(self, channel, author, params):
+    @command
+    def search(self, channel, author, params):
         """
         Searches all users to see who needs the given collectables. The names must match
         the in-game item name, and more than one can be provided if separated by commas.
@@ -1115,13 +1173,15 @@ class Turbot(discord.Client):
             lines.append(s("search_invalid", items=", ".join(sorted(invalid))))
         return "\n".join(sorted(lines)), None
 
-    def allfossils_command(self, channel, author, params):
+    @command
+    def allfossils(self, channel, author, params):
         """
         Shows all possible fossils that you can donate to the museum.
         """
         return s("allfossils", list=", ".join(sorted(FOSSILS_SET))), None
 
-    def uncollected_command(self, channel, author, params):
+    @command
+    def uncollected(self, channel, author, params):
         """
         Lists all collectables that you still need to donate. If a user is provided, it
         gives the same information for that user instead. | [user]
@@ -1173,7 +1233,8 @@ class Turbot(discord.Client):
 
         return "\n".join(lines), None
 
-    def neededfossils_command(self, channel, author, params):
+    @command
+    def neededfossils(self, channel, author, params):
         """
         Lists all the needed fossils for all the channel members.
         """
@@ -1200,7 +1261,8 @@ class Turbot(discord.Client):
             return s("neededfossils_none"), None
         return "\n".join(sorted(lines)), None
 
-    def collected_command(self, channel, author, params):
+    @command
+    def collected(self, channel, author, params):
         """
         Lists all collectables that you have already donated. If a user is provided, it
         gives the same information for that user instead. | [user]
@@ -1250,13 +1312,15 @@ class Turbot(discord.Client):
             )
         return "\n".join(lines), None
 
-    def predict_command(self, channel, author, params):
+    @command
+    def predict(self, channel, author, params):
         """
-        Get a link to a prediction calulator for a price history. | [user]
+        Get a link to a prediction calculator for a price history. | [user]
         """
         target = author.id if not params else params[0]
         target_name = discord_user_name(channel, target)
         target_id = discord_user_id(channel, target_name)
+        target_user = discord_user_from_id(channel, target_id)
         if not target_name or not target_id:
             return s("cant_find_user", name=target), None
 
@@ -1264,90 +1328,35 @@ class Turbot(discord.Client):
         if not timeline[0]:
             return s("cant_find_buy", name=target_name), None
 
+        self.generate_graph(channel, target_user, GRAPHCMD_FILE)
         query = ".".join((str(price) if price else "") for price in timeline).rstrip(".")
         url = f"{self.base_prophet_url}{query}"
-        return s("predict", name=target_name, url=url), None
+        return s("predict", name=target_name, url=url), discord.File(GRAPHCMD_FILE)
 
-    def friend_command(self, channel, author, params):
+    @command
+    def pref(self, channel, author, params):
         """
-        Set your friend code. | <code>
+        Set one of your user preferences. | <preference> <value>
         """
         if not params:
-            return s("friend_no_params"), None
+            return s("pref_no_params", prefs=", ".join(USER_PREFRENCES)), None
 
-        code = re.sub("[^0-9]", "", "".join(params).replace("-", ""))
-        if len(code) != 12 or not code.isdigit():
-            return s("friend_invalid"), None
+        pref = params[0].lower()
+        if pref not in USER_PREFRENCES:
+            return s("pref_invalid_pref", prefs=", ".join(USER_PREFRENCES)), None
+        if len(params) <= 1:
+            return s("pref_no_value", pref=pref), None
 
-        self.save_user_pref(author, "friend", code)
-        return s("friend", name=author), None
+        value = " ".join(params[1:])
+        validated_value = getattr(Validate, pref)(value)
+        if not validated_value:
+            return s(f"{pref}_invalid"), None
 
-    def fruit_command(self, channel, author, params):
-        """
-        Set your island's native fruit. | [apple|cherry|orange|peach|pear]
-        """
-        if not params:
-            return s("fruit_no_params"), None
+        self.save_user_pref(author, pref, validated_value)
+        return s("pref", pref=pref, name=author), None
 
-        fruit = params[0].lower()
-        if fruit not in ["apple", "cherry", "orange", "peach", "pear"]:
-            return s("fruit_invalid"), None
-
-        self.save_user_pref(author, "fruit", fruit)
-        return s("fruit", name=author), None
-
-    def hemisphere_command(self, channel, author, params):
-        """
-        Set your hemisphere. | [Northern|Southern]
-        """
-        if not params:
-            return s("hemisphere_no_params"), None
-
-        home = params[0].lower()
-        if home not in ["northern", "southern"]:
-            return s("hemisphere_bad_params"), None
-
-        self.save_user_pref(author, "hemisphere", home)
-        return s("hemisphere", name=author), None
-
-    def nickname_command(self, channel, author, params):
-        """
-        Set your nickname, such as your Switch user name. | <name>
-        """
-        if not params:
-            return s("nickname_no_params"), None
-
-        name = " ".join(params)  # allow spaces in nicknames
-        self.save_user_pref(author, "nickname", name)
-        return s("nickname", name=author), None
-
-    def timezone_command(self, channel, author, params):
-        """
-        Set your timezone. You can find a list of supported TZ names at
-        <https://en.wikipedia.org/wiki/List_of_tz_database_time_zones> | <zone>
-        """
-        if not params:
-            return s("timezone_no_params"), None
-
-        zone = params[0]
-        if zone not in pytz.all_timezones_set:
-            return s("timezone_bad_params"), None
-
-        self.save_user_pref(author, "timezone", zone)
-        return s("timezone", name=author), None
-
-    def island_command(self, channel, author, params):
-        """
-        Set your island name. | <name>
-        """
-        if not params:
-            return s("island_no_params"), None
-
-        island = " ".join(params)  # allow spaces in island names
-        self.save_user_pref(author, "island", island)
-        return s("island", name=author), None
-
-    def count_command(self, channel, author, params):
+    @command
+    def count(self, channel, author, params):
         """
         Provides a count of the number of pieces of collectables for the comma-separated
         list of users. | <list of users>
@@ -1395,7 +1404,8 @@ class Turbot(discord.Client):
 
         return "\n".join(lines), None
 
-    def art_command(self, channel, author, params):
+    @command
+    def art(self, channel, author, params):
         """
         Get info about pieces of art that are available | [List of art pieces]
         """
@@ -1514,7 +1524,8 @@ class Turbot(discord.Client):
         lines = [s(kind, **details(row)) for _, row in available.iterrows()]
         return "\n".join(add_header(sorted(lines)))
 
-    def fish_command(self, channel, author, params):
+    @command
+    def fish(self, channel, author, params):
         """
         Tells you what fish are available now in your hemisphere.
         | [name|leaving|arriving]
@@ -1524,7 +1535,8 @@ class Turbot(discord.Client):
             None,
         )
 
-    def bugs_command(self, channel, author, params):
+    @command
+    def bugs(self, channel, author, params):
         """
         Tells you what bugs are available now in your hemisphere.
         | [name|leaving|arriving]
@@ -1534,7 +1546,8 @@ class Turbot(discord.Client):
             None,
         )
 
-    def new_command(self, channel, author, params):
+    @command
+    def new(self, channel, author, params):
         """
         Tells you what new things available in your hemisphere right now.
         """
@@ -1567,14 +1580,23 @@ class Turbot(discord.Client):
         nickname = prefs.get("nickname", None)
         if nickname:
             embed.add_field(name="Nickname", value=nickname)
+        else:
+            embed.add_field(name="Nickname", value="Not set")
 
         code = prefs.get("friend", None)
         if code:
-            code_str = f"SW-{code[0:4]}-{code[4:8]}-{code[8:]}"
+            code_str = f"SW-{code[0:4]}-{code[4:8]}-{code[8:12]}"
             embed.add_field(name="Friend code", value=code_str)
         else:
             embed.add_field(name="Friend code", value="Not set")
 
+        code = prefs.get("creator", None)
+        if code:
+            code_str = f"MA-{code[0:4]}-{code[4:8]}-{code[8:12]}"
+            embed.add_field(name="Creator code", value=code_str)
+        else:
+            embed.add_field(name="Creator code", value="Not set")
+
         island = prefs.get("island", "Not set")
         embed.add_field(name="Island", value=island)
 
@@ -1590,7 +1612,8 @@ class Turbot(discord.Client):
 
         return embed
 
-    def info_command(self, channel, author, params):
+    @command
+    def info(self, channel, author, params):
         """
         Gives you information on a user. | [user]
         """
@@ -1601,7 +1624,7 @@ class Turbot(discord.Client):
 
         users = self.load_users()
         for _, row in users.iterrows():
-            user_id = row["author"]
+            user_id = int(row["author"])
             user_name = discord_user_name(channel, user_id)
             if not user_name:
                 continue
@@ -1616,7 +1639,8 @@ class Turbot(discord.Client):
 
         return s("info_not_found"), None
 
-    def about_command(self, channel, author, params):
+    @command
+    def about(self, channel, author, params):
         """
         Get information about Turbot.
         """
@@ -1721,6 +1745,12 @@ def get_channels(channels_file):  # pragma: no cover
     help="read users preferences data from this file",
 )
 @click.version_option(version=__version__)
+@click.option(
+    "--dev",
+    default=False,
+    is_flag=True,
+    help="Development mode, automatically reload bot when source changes",
+)
 def main(
     log_level,
     verbose,
@@ -1731,12 +1761,19 @@ def main(
     art_file,
     fossils_file,
     users_file,
+    dev,
 ):  # pragma: no cover
     auth_channels = get_channels(auth_channels_file) + list(channel)
     if not auth_channels:
         print("error: you must provide at least one authorized channel", file=sys.stderr)
         sys.exit(1)
 
+    if dev:
+        reloader = hupper.start_reloader("turbot.main")
+        reloader.watch_files(
+            [ART_DATA_FILE, BUGS_DATA_FILE, FISH_DATA_FILE, STRINGS_DATA_FILE]
+        )
+
     # ensure transient application directories exist
     DB_DIR.mkdir(exist_ok=True)
     TMP_DIR.mkdir(exist_ok=True)
